'=========================================================================================
'  BACK1 Remote HYPERTEM Operating System RNBFRA 1.2
'   Author   R.Toegel    Aug, 2005
'=========================================================================================
$regfile = "m32def.dat"
$hwstack = 128
$swstack = 256
$framesize = 48

$crystal = 8000000                                          'Quarzfrequenz
$baud = 9600                                                'Baudrate fr RS232

Const Unit_c_top = 32                                       ' max number of units
Const Tim_c_top = 16                                        ' max timer
Const Msg_c_top = 48                                        ' max msg

'=========================================================================================
'  constant definition   onboard I2C Addresses
'=========================================================================================
Const Pwr_adr = &H74                                        'I2C Adress Powerport
Dim Pwrbyte As Byte                                         ' PCF power
'  7   6   5   4  3   2   1    0
' LED LED LED  x  x  5V SLEEP MOT

Dim Pwrsel As Byte                                          ' PCF bit

'=========================================================================================
Const Out_adr = &H72                                        'I2C Adress Extension-Out Port
Dim Outbyte As Byte                                         ' PCF out
'  7   6   5   4  3   2   1   0
'  x   x   x   x LED LED LED LED

Dim Outsel As Byte                                          ' PCF bit

'=========================================================================================
Const Exp_adr = &H7F                                        'I2C Read Adress Expansion-In Port

Dim Expbyte As Byte                                         ' PCF input
'  7   6   5   4  3   2   1   0
'  x   x   x   x  x   x   x   x



'=========================================================================================
Const Tmp_adr = &H90                                        'I2C Adress temperature LM75
Const Srv_adr = &HC2                                        'I2C Adress SD21
Const Sf8_adr = &HE0                                        'I2C Adress SF 08 1

'=========================================================================================
'  Functions & Subs I2C Bus
'=========================================================================================
Config I2cdelay = 5
Config Scl = Portc.0                                        'Ports fuer IIC-Bus
Config Sda = Portc.1
'=========================================================================================

'=========================================================================================
'  Functions & Subs UART TX
'=========================================================================================
Config Serialout = Buffered , Size = 64                     'RS232 Out buffered

Dim Buffill As Byte                                         ' current fill of serialout

'=========================================================================================
'  Functions & Subs UART RX
'=========================================================================================
Declare Sub Rxreceive()

Config Serialin = Buffered , Size = 64                      'RS232 Inp buffered

Dim Rxflag As Byte                                          ' excape-Sequence
Dim Hilight As Byte                                         ' focus device


'=========================================================================================
'  useful fields
'=========================================================================================
Dim Tempb As Byte                                           ' current fill of serialout
Dim Refresh As Byte                                         ' text refresh device
Dim Absi As Integer                                         ' work integer

Dim Pwrunit As Byte                                         ' PCF power
Dim Outunit As Byte                                         ' PCF power




'=========================================================================================

Config Adc = Single , Prescaler = Auto

'=========================================================================================
' Timer0 Prescale for 10mS Interrupt
'=========================================================================================
Config Timer0 = Timer , Prescale = 64                       'Timer 1mS
Const Tmr_c_preload = 131
On Timer0 Interrupt_ticker                                  ' Timer for Timer Queue

'=========================================================================================
'  Commands
'=========================================================================================
Const Cmd_c_setup = &H80                                    ' Setup device I
Const Cmd_c_time = &H81
Const Cmd_c_show = &H82
Const Cmd_c_incr = &H83
Const Cmd_c_decr = &H84
Const Cmd_c_set = &H85
Const Cmd_c_text = &H86
Const Cmd_c_start = &H87
Const Cmd_c_on = &H88
Const Cmd_c_off = &H89
Const Cmd_c_togg = &H8A


'=========================================================================================
' Classes ID Sense
'=========================================================================================
Const Cls_c_sys = &H01
Const Cls_c_adc = &H02
Const Cls_c_batt = &H03
Const Cls_c_gp12 = &H05
Const Cls_c_expand = &H04

'=========================================================================================
' Classes ID Act
'=========================================================================================
Const Cls_c_servo = &H83
Const Cls_c_stepp = &H84
Const Cls_c_gears = &H85
Const Cls_c_pcfpwr = &H81
Const Cls_c_pcfout = &H82

'=========================================================================================
' UNIT ENTRY Data
'=========================================================================================
Dim Unitfill As Byte                                        ' top unit entry
Dim Unitflag(unit_c_top) As Byte                            ' controlling flag
Dim Unitrefcls(unit_c_top) As Byte                          ' Unit Class
Dim Unitrefid(unit_c_top) As Byte                           ' Unit Ident


'=========================================================================================
' UNIT Sensor Data
'=========================================================================================
Const Sense_c_top = 18                                      ' max sensor
Dim Sensefill As Byte
Dim Senseref(sense_c_top) As Byte                           ' backlink unit
Dim Senseval(sense_c_top) As Word                           ' current value



'=========================================================================================
'  Timer 1 mS  Signal to main loop
'=========================================================================================
Dim Irptflag As Byte                                        ' Timer Interrupt occurred

'=========================================================================================
'  Timer Entries
'=========================================================================================
Dim Maxtim As Byte
Dim Curtim As Byte

Dim Timentflag(tim_c_top) As Byte
Dim Timentrefix(tim_c_top) As Byte                          ' referenced Unit Index
Dim Timentrefcmd(tim_c_top) As Byte                         ' backlink reference #
Dim Timentcounter(tim_c_top) As Word                        ' Number of ticks to wait

Declare Sub Timertick()
Declare Function Timentadd(byval Idx As Byte , Byval Ref As Byte , Byval Count As Word ) As Byte

'=========================================================================================
'  Message Queue
'=========================================================================================
Dim Maxmsg As Byte
Dim Curmsg As Byte

Dim Msgpntr As Byte                                         ' queue working Index#
Dim Msgentflag(msg_c_top) As Byte
Dim Msgenttgtix(msg_c_top) As Byte                          ' forward UnitID Index
Dim Msgenttgtcmd(msg_c_top) As Byte                         ' forward Command
Dim Msgenttgtpar(msg_c_top) As Word                         ' backlink UnitID Index

Declare Function Msgentadd(byval Tgtix As Byte , Byval Tgtcmd As Byte , Byval Param As Word) As Byte
Declare Sub Msgcall(byval Tgtix As Byte , Byval Tgtcmd As Byte , Byval Param As Word)





'=========================================================================================
'  Device Class Sysbeat
'=========================================================================================
Const Rfsh_c_sys = 1000                                     ' 1 sec refresh
Const Rfsh_c_text = 9788                                    ' 9.788 sec refresh screen
'=========================================================================================
Dim Clock_hh As Byte
Dim Clock_mm As Byte
Dim Clock_ss As Byte

Declare Sub Sysbuild()
Declare Sub Sys(byval Unitx As Byte , Byval Cmd As Byte , Byval Param As Word)


'=========================================================================================
' ADC Class
'=========================================================================================
Const Adc_c_fact = 4.8876
Const Rfsh_c_adc = 2000                                     ' 2 sec refresh
Declare Sub Adcbuild()
Declare Sub Adc(byval Unitx As Byte , Byval Cmd As Byte , Byval Param As Word)

'=========================================================================================
' GP2D12 (ADC) Class
'=========================================================================================
Const Rfsh_c_gp12 = 100                                     ' 100ms sec refresh
Declare Sub Gp12build()
Declare Sub Gp12(byval Unitx As Byte , Byval Cmd As Byte , Byval Param As Word)

'=========================================================================================
' Battery Class
'=========================================================================================
Const Batt_c_fact = 21                                      ' adcval * fact ==> send-value
'=========================================================================================
Const Rfsh_c_batt = 1500                                    ' 2 sec refresh
Declare Sub Battbuild()
Declare Sub Batt(byval Unitx As Byte , Byval Cmd As Byte , Byval Param As Word)

'=========================================================================================
'  Expansion port Class
'=========================================================================================
Const Rfsh_c_expand = 900                                   ' 0.9 sec refresh
Declare Sub Expandbuild()
Declare Sub Expand(byval Unitx As Byte , Byval Cmd As Byte , Byval Param As Word)

'=========================================================================================
'  Powerport Class
'=========================================================================================
Declare Sub Pwrportbuild()
Declare Sub Pwrport(byval Unitx As Byte , Byval Cmd As Byte , Byval Param As Word)

'=========================================================================================
'  Outport Class
'=========================================================================================
Declare Sub Outportbuild()
Declare Sub Outport(byval Unitx As Byte , Byval Cmd As Byte , Byval Param As Word)

'=========================================================================================
' Servo Class
'=========================================================================================
Const Servo_c_min = 600                                     ' servo position left
Const Servo_c_mid = 1300                                    ' servo position middle
Const Servo_c_max = 2000                                    ' servo position right

Const Servo_c_top = 10                                      ' max servos
Dim Servofill As Byte
Dim Servoref(servo_c_top) As Byte                           ' backlink unit
Dim Servoval(servo_c_top) As Word                           ' position

'=========================================================================================
Declare Sub Servobuild()
Declare Sub Servo(byval Unitx As Byte , Byval Cmd As Byte , Byval Param As Word)

'=========================================================================================
' Stepper Class
'=========================================================================================
Const Stepp_c_top = 2                                       ' max stepper
Dim Steppfill As Byte
Dim Steppref(stepp_c_top) As Byte                           ' backlink unit

Dim Steppspeedrite As Integer                               ' speed right
Dim Steppspeedleft As Integer                               ' speed left
Dim Steppcountrite As Byte                                  ' run divisor
Dim Steppcountleft As Byte                                  ' run divisor

Dim Steppspeed(stepp_c_top) As Integer At Steppspeedrite Overlay       ' speed
Dim Steppcount(stepp_c_top) As Byte At Steppcountrite Overlay       ' run divisor

'=========================================================================================
Declare Sub Steppbuild()
Declare Sub Stepp(byval Unitx As Byte , Byval Cmd As Byte , Byval Param As Word)



'=========================================================================================
' Gears Class
'=========================================================================================
Const Gears_c_top = 2                                       ' max gear mot
Dim Gearsfill As Byte
Dim Gearsref(gears_c_top) As Byte                           ' backlink unit

Dim Gearsspeedleft As Integer
Dim Gearsspeedrite As Integer
Dim Gearsspeed(gears_c_top) As Integer At Gearsspeedleft Overlay       ' speed

'=========================================================================================
Declare Sub Gearsbuild()
Declare Sub Gears(byval Unitx As Byte , Byval Cmd As Byte , Byval Param As Word)


'==============================================================================
'  END of DATA
'==============================================================================






'==============================================================================
'  M A I N    S T A R T U P
'==============================================================================
   Gosub Clearscreen
   For Msgpntr = 5 To 1 Step -1
      Print "-" ; Hex(msgpntr)
      Wait 1
   Next
   Gosub Clearscreen

   Msgpntr = 1
'=========================================================================================
' Build Unit
'=========================================================================================
   Call Sysbuild()
'=========================================================================================
   Call Expandbuild()
'=========================================================================================
   Call Gp12build()
'=========================================================================================
   Call Adcbuild()
   Call Adcbuild()
   Call Adcbuild()
'=========================================================================================
   Call Battbuild()                                         ' battery check
'=========================================================================================
   Call Adcbuild()
   Call Adcbuild()
   Call Adcbuild()
'=========================================================================================
   Call Servobuild()
   Call Servobuild()
   Call Servobuild()
   Call Servobuild()
   Call Servobuild()
   Call Servobuild()
'=========================================================================================
   Call Gearsbuild()                                        ' option 1
   Call Gearsbuild()                                        ' option 1
'=========================================================================================
   Call Steppbuild()                                        ' option 2
   Call Steppbuild()
'=========================================================================================
   Call Pwrportbuild()
   Pwrunit = Unitfill
   Call Outportbuild()
   Outunit = Unitfill
'=========================================================================================


   Gosub Hilight_next

   Refresh = Msgentadd(1 , Cmd_c_start , 0)
'=========================================================================================
   I2cinit                                                  'I2C Initialisieren


   Start Adc                                                ' Init Analog-Digital-Converter



   Enable Interrupts                                        'General enable

'==============================================================================
'  M A I N    L O O P
'==============================================================================
Do
      If Irptflag = 1 Then
         Irptflag = 0
         Gosub Stepptick1
         Gosub Stepptick2
         Call Timertick()                                   ' check timer entries
      End If
      Gosub Getfill
'===== print-buffer space ?
      If Buffill < 8 Then
         If Ischarwaiting() = 1 Then                        ' if data-> workout
            Call Rxreceive()
         End If
         If Msgentflag(msgpntr) = 1 Then                    ' active Entry ?
            Call Msgcall(msgenttgtix(msgpntr) , Msgenttgtcmd(msgpntr) , Msgenttgtpar(msgpntr))
            Msgentflag(msgpntr) = 0
            Decr Curmsg
         End If
         Incr Msgpntr
         If Msgpntr > Msg_c_top Then Msgpntr = 1
      End If
  Loop
 End
'==============================================================================
' M A I N   L O O P   E N D
'==============================================================================


'==============================================================================
' Timer 0  interrupt
'==============================================================================
Interrupt_ticker:
   Timer0 = Tmr_c_preload
   Irptflag = 1
Return


'==============================================================================
'  Message Queue Functions     Call
'==============================================================================
Sub Msgcall(byval Tgtix As Byte , Byval Tgtcmd As Byte , Byval Param As Word )
   If Tgtix > 0 Then
      Select Case Unitrefcls(tgtix)
      Case Cls_c_sys : Call Sys(tgtix , Tgtcmd , Param)
      Case Cls_c_expand : Call Expand(tgtix , Tgtcmd , Param)
      Case Cls_c_pcfpwr : Call Pwrport(tgtix , Tgtcmd , Param)
      Case Cls_c_pcfout : Call Outport(tgtix , Tgtcmd , Param)
      Case Cls_c_adc : Call Adc(tgtix , Tgtcmd , Param)
      Case Cls_c_gp12 : Call Gp12(tgtix , Tgtcmd , Param)
      Case Cls_c_batt : Call Batt(tgtix , Tgtcmd , Param)
      Case Cls_c_servo : Call Servo(tgtix , Tgtcmd , Param)
      Case Cls_c_gears : Call Gears(tgtix , Tgtcmd , Param)
      Case Cls_c_stepp : Call Stepp(tgtix , Tgtcmd , Param)
      End Select
   End If
End Sub
'==============================================================================
'  Message Queue Functions     Add Entry
'==============================================================================
Function Msgentadd(byval Tgtix As Byte , Byval Tgtcmd As Byte , Byval Param As Byte ) As Byte
   For Msgentadd = Msgpntr To Msg_c_top
      If Msgentflag(msgentadd) = 0 Then                     ' free entry
         Gosub Msginsert
         Exit Function
      End If
   Next
   For Msgentadd = 1 To Msgpntr
      If Msgentflag(msgentadd) = 0 Then                     ' free entry
         Gosub Msginsert
         Exit Function
      End If
   Next
   Msgentadd = 0
   Exit Function
Msginsert:
   Msgenttgtix(msgentadd) = Tgtix
   Msgenttgtcmd(msgentadd) = Tgtcmd
   Msgenttgtpar(msgentadd) = Param
   Msgentflag(msgentadd) = 1                                ' set active
   Incr Curmsg
   If Maxmsg < Curmsg Then Maxmsg = Curmsg
   Return
End Function

'==============================================================================
'  Timer Queue Functions     Timer Tick
'==============================================================================
Sub Timertick()
Local Pntr As Byte
Local Mts As Byte
      For Pntr = 1 To Tim_c_top
         If Timentflag(pntr) = 1 Then                       ' active ?
            If Timentcounter(pntr) > 0 Then
               Decr Timentcounter(pntr)
            End If
            If Timentcounter(pntr) = 0 Then
               Mts = Msgentadd(timentrefix(pntr) , Timentrefcmd(pntr) , 0 )
               If Mts > 0 Then
                  Timentflag(pntr) = 0                      ' set free
                  Decr Curtim
               End If
            End If
         End If
      Next
End Sub
'==============================================================================
'  Timer Queue Functions     Add Entry
'==============================================================================
Function Timentadd(byval Idx As Byte , Byval Ref As Byte , Byval Count As Word ) As Byte
Local Pntr As Byte
For Pntr = 1 To Tim_c_top
   If Timentflag(pntr) = 0 Then                             ' free entry
      Timentrefix(pntr) = Idx
      Timentrefcmd(pntr) = Ref
      Timentcounter(pntr) = Count
      Timentadd = Pntr
      Timentflag(pntr) = 1                                  'set active
      Incr Curtim
      If Maxtim < Curtim Then Maxtim = Curtim
      Exit Function
   End If
Next
   Timentadd = 0
End Function




'==============================================================================
' RX Function read input
'==============================================================================
Sub Rxreceive()
Local Rxchar As Byte
Local Ix As Byte
   While Ischarwaiting() <> 0
      Rxchar = Inkey()
      Select Case Rxflag
      Case 0:
         Select Case Rxchar
         Case 13:
            Ix = Msgentadd(hilight , Cmd_c_togg , 0)
         Case 27:
            Rxflag = 1
         Case Else:
            Print Chr(7) ;                                  ' click or beep
         End Select
      Case 1:
         If Rxchar = "[" Then
            Rxflag = 2                                      ' <ESC>[
         Else
            Rxflag = 0
         End If
      Case 2:
         Select Case Rxchar
         Case "A":                                          ' Arr UP
            Ix = Msgentadd(hilight , Cmd_c_text , 0)
            Gosub Hilight_prev
            Ix = Msgentadd(hilight , Cmd_c_text , 0)
         Case "B":                                          ' Arr DOWN
            Ix = Msgentadd(hilight , Cmd_c_text , 0)
            Gosub Hilight_next
            Ix = Msgentadd(hilight , Cmd_c_text , 0)
         Case "C":                                          ' Arr RIGHT
            Ix = Msgentadd(hilight , Cmd_c_incr , 0)
          Case "D":                                         ' Arr LEFT
            Ix = Msgentadd(hilight , Cmd_c_decr , 0)
         End Select
         Rxflag = 0
         Exit Sub
      End Select
   Wend
End Sub

Hilight_next:
   Incr Hilight
   For Hilight = Hilight To Unitfill
      If Unitrefcls(hilight) >= &H80 Then Return
   Next
   For Hilight = 1 To Unitfill
      If Unitrefcls(hilight) >= &H80 Then Return
   Next
   Return
Hilight_prev:
   Decr Hilight
   For Hilight = Hilight To 1 Step -1
      If Unitrefcls(hilight) >= &H80 Then Return
   Next
   For Hilight = Unitfill To 1
      If Unitrefcls(hilight) >= &H80 Then Return
   Next
   Return

'=========================================================================================
'   Get Fill of Out Buffer
'=========================================================================================
Getfill:
   Buffill = _rs_tail_ptrw0 Or &H40
   Buffill = Buffill - _rs_head_ptrw0
   Buffill = Buffill And &H3F
   Return
'=========================================================================================
'   Set graphic rendition
'=========================================================================================
Setrevers:
      Print "{027}[" ; "7m" ;
      Return
Setnormal:
      Print "{027}[" ; "0m" ;
      Return
Clearscreen:
   Print "{027}[";"1;1H";                                      ' home
   Gosub Setnormal
   Print "{027}[";"2J";                                        ' clear screen
   Print "{015}{027}[4l";                                   ' replace mode
   Print "{027}[";"?25l";                                      ' cursor disable
   Return
'=========================================================================================
' UNIT Work
'=========================================================================================
Unitcreate:
   Incr Unitfill
   Unitflag(unitfill) = 1                                   ' active
   Refresh = Msgentadd(unitfill , Cmd_c_setup , 0)
   Return

Sensecreate:
   Incr Sensefill
   Unitrefid(unitfill) = Sensefill
   Senseref(sensefill) = Unitfill
   Senseval(sensefill) = 0
   Return

'=========================================================================================
'  CLASS Sysbeat Clock
'=========================================================================================
Sub Sysbuild()
   Gosub Unitcreate
   Unitrefcls(unitfill) = Cls_c_sys
   Unitrefid(unitfill) = 0
End Sub
'=========================================================================================
Sub Sys(byval Unitx As Byte , Byval Cmd As Byte , Byval Param As Word)
Local Ix As Byte
Local Iy As Byte
Local Rxstr As String * 4
   Select Case Cmd
   Case Cmd_c_setup:
      Ix = Timentadd(unitx , Cmd_c_time , Rfsh_c_sys)
      Print "Setup: " ; Str(unitx) ; Spc(6);
      Print "SYS Clock  "
   Case Cmd_c_time:
      Incr Clock_ss
      If Clock_ss > 59 Then
         Clock_ss = 0
         Incr Clock_mm
         If Clock_mm > 59 Then
            Clock_mm = 0
            Incr Clock_hh
            If Clock_hh > 23 Then
               Clock_hh = 0
            End If
         End If
      Else
         Incr Refresh
         If Refresh > Unitfill Then
            Refresh = 1
         End If
         Ix = Msgentadd(refresh , Cmd_c_text , 0)
         Ix = Msgentadd(refresh , Cmd_c_show , 0)
      End If
      Ix = Timentadd(unitx , Cmd_c_time , Rfsh_c_sys)
      Ix = Msgentadd(unitx , Cmd_c_show , 0)
   Case Cmd_c_start:
      Print "{017}{027}[?25h";                              ' cursor enable
      Input "<ENTER>" , Rxstr
      Gosub Clearscreen
      Print "{009}{009}{009}{009}R N B F R A  1.2"
      Waitms 100
      Print "{027}[";"21;1HSelect ==> ARR UP/DWN";
      Waitms 100
      Print "{027}[";"22;1HValue +/- ==> ARR Left/Right";
      Waitms 100
      Print "{027}[";"23;1HBit Toggle ==> ENTER";
      Waitms 100

      Enable Timer0
      Start Timer0                                          ' timer starten

   Case Cmd_c_show:
      Print "{027}[";"1;70H" ;
      If Clock_hh < 10 Then Printbin 48
      Print Str(clock_hh) ; ":";
      If Clock_mm < 10 Then Printbin 48
      Print Str(clock_mm) ; ":";
      If Clock_ss < 10 Then Printbin 48
      Print Str(clock_ss);
      Print "{027}[";"2;70HMSG:" ; Str(curmsg) ; "/" ; Str(maxmsg) ; Spc(5);
      Print "{027}[";"3;70HTIM:" ; Str(curtim) ; "/" ; Str(maxtim) ; Spc(5);

   End Select
End Sub
'=========================================================================================
'  CLASS expansionport
'=========================================================================================
Sub Expandbuild()
   Gosub Unitcreate
   Unitrefcls(unitfill) = Cls_c_expand
   Unitrefid(unitfill) = 0
End Sub
'=========================================================================================
Sub Expand(byval Unitx As Byte , Byval Cmd As Byte , Byval Param As Word)
Local Ix As Byte
Local Iy As Byte
   Select Case Cmd
   Case Cmd_c_setup:
      Ix = Msgentadd(unitx , Cmd_c_text , 0)
      Ix = Timentadd(unitx , Cmd_c_time , Rfsh_c_expand)
      Print "Setup: " ; Str(unitx) ; Spc(6);
      Ix = Exp_adr - 1
      Print "EXPAND     I2C=&H" ; Hex(ix)
   Case Cmd_c_text:
      Print "{027}[";"13;1HInPort:";
   Case Cmd_c_time:
      I2cstart
      I2cwbyte Exp_adr
      I2crbyte Expbyte , Nack
      I2cstop
      Ix = Msgentadd(unitx , Cmd_c_show , 0)
      Ix = Timentadd(unitx , Cmd_c_time , Rfsh_c_expand)
   Case Cmd_c_show:
      Print "{027}[";"13;8H" ; Bin(expbyte);
   End Select
End Sub

'=========================================================================================
'  CLASS Power Port
'=========================================================================================
Sub Pwrportbuild()
   Gosub Unitcreate
   Unitrefcls(unitfill) = Cls_c_pcfpwr
   Unitrefid(unitfill) = 0
End Sub
'=========================================================================================
Sub Pwrport(byval Unitx As Byte , Byval Cmd As Byte , Byval Param As Word)
Local Ix As Byte
Local Iy As Byte
   Select Case Cmd
   Case Cmd_c_setup:
      Ix = Msgentadd(unitx , Cmd_c_text , 0)
      Print "Setup: " ; Str(unitx) ; Spc(6);
      Pwrbyte = &B01100010
      Ix = Msgentadd(unitx , Cmd_c_set , 0)
      Ix = Pwr_adr
      Print "POWER-Port I2C=&H" ; Hex(ix)
   Case Cmd_c_on:
      Pwrbyte = Pwrbyte Or Param
      Ix = Msgentadd(unitx , Cmd_c_set , 0)
   Case Cmd_c_off:
      Ix = Param Xor &HFF
      Pwrbyte = Pwrbyte And Ix
      Ix = Msgentadd(unitx , Cmd_c_set , 0)
   Case Cmd_c_set:
      I2cstart
      I2cwbyte Pwr_adr
      I2cwbyte Pwrbyte
      I2cstop
      Ix = Msgentadd(unitx , Cmd_c_show , 0)
   Case Cmd_c_text:
      If Unitx = Hilight Then Gosub Setrevers
      Print "{027}[";"17;20HPower:";
      If Unitx = Hilight Then Gosub Setnormal
   Case Cmd_c_decr:
         If Pwrsel < 7 Then Incr Pwrsel
         Ix = Msgentadd(unitx , Cmd_c_show , 0)
   Case Cmd_c_incr:
         If Pwrsel > 0 Then Decr Pwrsel
         Ix = Msgentadd(unitx , Cmd_c_show , 0)
   Case Cmd_c_togg:
         Toggle Pwrbyte.pwrsel
         Ix = Msgentadd(unitx , Cmd_c_set , 0)
   Case Cmd_c_show:
      Print "{027}[";"17;28H" ;
      For Ix = 7 To 0 Step -1
         If Ix = Pwrsel Then Gosub Setrevers
         If Pwrbyte.ix = 1 Then
               Print "1";
         Else
               Print "0";
         End If
         If Ix = Pwrsel Then Gosub Setnormal
      Next
   End Select
End Sub

'=========================================================================================
'  CLASS OUTPT Port
'=========================================================================================
Sub Outportbuild()
   Gosub Unitcreate
   Unitrefcls(unitfill) = Cls_c_pcfout
   Unitrefid(unitfill) = 0
End Sub
'=========================================================================================
Sub Outport(byval Unitx As Byte , Byval Cmd As Byte , Byval Param As Word)
Local Ix As Byte
Local Iy As Byte
   Select Case Cmd
   Case Cmd_c_setup:
      Ix = Msgentadd(unitx , Cmd_c_text , 0)
      Print "Setup: " ; Str(unitx) ; Spc(6);
      Outbyte = &B00001100
      Ix = Msgentadd(unitx , Cmd_c_set , 0)
      Ix = Out_adr
      Print "OUTPT-Port I2C=&H" ; Hex(ix)
   Case Cmd_c_on:
      Outbyte = Outbyte Or Param
      Ix = Msgentadd(unitx , Cmd_c_set , 0)
   Case Cmd_c_off:
      Ix = Param Xor &HFF
      Outbyte = Outbyte And Ix
      Ix = Msgentadd(unitx , Cmd_c_set , 0)
   Case Cmd_c_set:
      I2cstart
      I2cwbyte Out_adr
      I2cwbyte Outbyte
      I2cstop
      Ix = Msgentadd(unitx , Cmd_c_show , 0)
   Case Cmd_c_text:
      If Unitx = Hilight Then Gosub Setrevers
      Print "{027}[";"18;20HOutpt:";
      If Unitx = Hilight Then Gosub Setnormal
   Case Cmd_c_decr:
         If Outsel < 7 Then Incr Outsel
         Ix = Msgentadd(unitx , Cmd_c_show , 0)
   Case Cmd_c_incr:
         If Outsel > 0 Then Decr Outsel
         Ix = Msgentadd(unitx , Cmd_c_show , 0)
   Case Cmd_c_togg:
         Toggle Outbyte.outsel
         Ix = Msgentadd(unitx , Cmd_c_set , 0)
   Case Cmd_c_show:
      Print "{027}[";"18;28H" ;
      For Ix = 7 To 0 Step -1
         If Ix = Outsel Then Gosub Setrevers
         If Outbyte.ix = 1 Then
               Print "1";
         Else
               Print "0";
         End If
         If Ix = Outsel Then Gosub Setnormal
      Next
   End Select
End Sub





'=========================================================================================
' CLASS  ADC 5V Ref
'=========================================================================================
Sub Adcbuild()
   Gosub Unitcreate
   Unitrefcls(unitfill) = Cls_c_adc
   Gosub Sensecreate
End Sub
'=========================================================================================
Sub Adc(byval Unitx As Byte , Byval Cmd As Byte , Byval Param As Word)
Local Ix As Byte
Local Chn As Byte
Local Currw As Word
Local Currs As Single
   Chn = Unitrefid(unitx)
   Select Case Cmd
   Case Cmd_c_setup:
      Currw = Chn * 10
      Currw = Currw + Rfsh_c_adc
      Ix = Msgentadd(unitx , Cmd_c_text , 0)
      Ix = Timentadd(unitx , Cmd_c_time , Currw)
      Print "Setup: " ; Str(unitx) ; Spc(6);
      Print "ADC 5V    " ; Hex(chn)
   Case Cmd_c_time:
      Ix = Chn - 1
      Currw = Getadc(ix)
      Senseval(chn) = Currw
      Ix = Timentadd(unitx , Cmd_c_time , Rfsh_c_adc)
      Ix = Msgentadd(unitx , Cmd_c_show , 0)
   Case Cmd_c_text:
      Ix = Chn + 1
      Print "{027}[" ; Str(ix) ; ";1HADC" ; Hex(chn) ; ":" ;
   Case Cmd_c_show:
      Currs = Senseval(chn) * Adc_c_fact
      Currs = Round(currs)
      Currw = Currs
      Ix = Chn + 1
      Print "{027}[" ; Str(ix) ; ";7H" ; Str(currw) ; " mV    ";
   End Select
End Sub
'=========================================================================================
' CLASS Adc Battery Check
'=========================================================================================
Sub Battbuild()
   Gosub Unitcreate
   Unitrefcls(unitfill) = Cls_c_batt
   Gosub Sensecreate
End Sub
'=========================================================================================
Sub Batt(byval Unitx As Byte , Byval Cmd As Byte , Byval Param As Word)
Local Ix As Byte
Local Chn As Byte
Local Currw As Word
   Chn = Unitrefid(unitx)
   Select Case Cmd
   Case Cmd_c_setup:
      Ix = Timentadd(unitx , Cmd_c_time , Rfsh_c_batt)
      Ix = Msgentadd(unitx , Cmd_c_text , 0)
      Print "Setup: " ; Str(unitx) ; Spc(6);
      Print "Battery   " ; Hex(chn)
   Case Cmd_c_time:
      Ix = Chn - 1
      Currw = Getadc(ix)
      Senseval(chn) = Currw
      Ix = Timentadd(unitx , Cmd_c_time , Rfsh_c_batt)
      Ix = Msgentadd(unitx , Cmd_c_show , 0)
   Case Cmd_c_text:
      Ix = Chn + 1
      Print "{027}[" ; Str(ix) ; ";1HBATT: " ;
   Case Cmd_c_show:
      Currw = Senseval(chn) * Batt_c_fact
      Ix = Chn + 1
      Print "{027}[" ; Str(ix) ; ";6H" ; Str(currw) ; " mV    ";
   End Select
End Sub
'=========================================================================================
'  CLASS  SHARP  GP2D12
'=========================================================================================
Sub Gp12build()
   Gosub Unitcreate
   Unitrefcls(unitfill) = Cls_c_gp12
   Gosub Sensecreate
End Sub
'=========================================================================================
Sub Gp12(byval Unitx As Byte , Byval Cmd As Byte , Byval Param As Word)
Local Ix As Byte
Local Iy As Byte
Local Chn As Byte
Local Currw As Word
Local Currp As Word
   Chn = Unitrefid(unitx)
   Select Case Cmd
   Case Cmd_c_setup:
      Ix = Msgentadd(unitx , Cmd_c_text , 0)
      Ix = Timentadd(unitx , Cmd_c_time , Rfsh_c_gp12)
      Print "Setup: " ; Str(unitx) ; Spc(6);
      Print "GP2D12    " ; Hex(chn)
   Case Cmd_c_time:
      Ix = Chn - 1
      Currp = Senseval(chn)
      Currw = Getadc(ix)
      Currw = Currw And &H7FF8
      If Currp <> Currw Then
         Senseval(chn) = Currw
         Ix = Msgentadd(unitx , Cmd_c_show , 0)
      End If
      Ix = Timentadd(unitx , Cmd_c_time , Rfsh_c_gp12)
   Case Cmd_c_text:
      Ix = Chn + 1
      Print "{027}[" ; Str(ix) ; ";1HGP-" ; Hex(chn) ; ":" ;
   Case Cmd_c_show:
      Currw = Senseval(chn)
      Ix = Chn + 1
      Print "{027}[" ; Str(ix) ; ";8H" ; Str(currw) ; "    ";
   End Select
End Sub
'=========================================================================================
' CLASS Servo
'=========================================================================================
Sub Servobuild()
   Gosub Unitcreate
   Unitrefcls(unitfill) = Cls_c_servo
   Incr Servofill
   Unitrefid(unitfill) = Servofill
   Servoref(servofill) = Unitfill
   Servoval(servofill) = Servo_c_mid
End Sub
'=========================================================================================
Sub Servo(byval Unitx As Byte , Byval Cmd As Byte , Byval Param As Word)
Local Ix As Byte
Local Chn As Byte
Local Currw As Word
   Chn = Unitrefid(unitx)
   Select Case Cmd
   Case Cmd_c_setup:
      Ix = Msgentadd(unitx , Cmd_c_text , 0)
      Ix = Msgentadd(unitx , Cmd_c_set , 0)
      Print "Setup: " ; Str(unitx) ; Spc(6);
      Print "Servo-" ; Hex(chn)
   Case Cmd_c_set:
      Currw = Servoval(chn)
      Ix = Chn + 64
      Print "{027}[";"20;1HCoProc:" ; "#s" ; Chr(ix);
      Printbin Currw
      Ix = Currw
      Print "{015}{017}";
      Ix = Msgentadd(unitx , Cmd_c_show , 0)
   Case Cmd_c_text:
      Ix = Chn + 1
      Print "{027}[" ; Str(ix) ; ";20H";
      If Unitx = Hilight Then Gosub Setrevers
      Print "Servo-" ; Hex(chn) ; ":" ;
      If Unitx = Hilight Then Gosub Setnormal
      Ix = Msgentadd(unitx , Cmd_c_show , 0)
   Case Cmd_c_show:
      Currw = Servoval(chn)
      Ix = Chn + 1
      Print "{027}[" ; Str(ix) ; ";29H" ; Str(currw) ; Spc(4);
   Case Cmd_c_decr:
      Currw = Servoval(chn)
      If Currw > Servo_c_min Then
         Currw = Currw - 10
         Servoval(chn) = Currw
      End If
      Ix = Msgentadd(unitx , Cmd_c_set , 0)
   Case Cmd_c_incr:
      Currw = Servoval(chn)
      If Currw < Servo_c_max Then
         Currw = Currw + 10
         Servoval(chn) = Currw
      End If
      Ix = Msgentadd(unitx , Cmd_c_set , 0)
   End Select
End Sub





'=========================================================================================
' CLASS Gears
'=========================================================================================
Sub Gearsbuild()
   Gosub Unitcreate
   Unitrefcls(unitfill) = Cls_c_gears
   Incr Gearsfill
   Unitrefid(unitfill) = Gearsfill
   Gearsref(gearsfill) = Unitfill
   Gearsspeed(gearsfill) = 0
   If Gearsfill = 1 Then
'=========================================================================================
'  Gear Motor Def
'=========================================================================================
'Ports Left Motor
Config Pinc.6 = Output                                      'Linker Motor Kanal 1
Config Pinc.7 = Output                                      'Linker Motor Kanal 2
Config Pind.4 = Output                                      'Linker Motor PWM
   Else
'Ports Right Motor
Config Pinb.0 = Output                                      'Rechter Motor Kanal 1
Config Pinb.1 = Output                                      'Rechter Motor Kanal 2
Config Pind.5 = Output                                      'Rechter Motor PWM
   End If
Config Timer1 = Pwm , , Prescale = 64 , Pwm = 10 , Compare A Pwm = Clear Down , Compare B Pwm = Clear Down
   Pwm1a = 0
   Pwm1b = 0


End Sub
'=========================================================================================
Sub Gears(byval Unitx As Byte , Byval Cmd As Byte , Byval Param As Word)
Local Ix As Byte
Local Chn As Byte
Local Currw As Integer
   Chn = Unitrefid(unitx)
   Select Case Cmd
   Case Cmd_c_setup:
      Ix = Msgentadd(unitx , Cmd_c_text , 0)
      Ix = Msgentadd(unitx , Cmd_c_set , 0)
      Print "Setup: " ; Str(unitx) ; Spc(6);
      Print "MOT " ; "Gear  " ; Hex(chn)
   Case Cmd_c_set:
      If Chn = 1 Then Gosub Gearscontrol1
      If Chn = 2 Then Gosub Gearscontrol2
      Ix = Msgentadd(unitx , Cmd_c_show , 0)
   Case Cmd_c_text:
      Ix = Chn + 12
      Print "{027}[" ; Str(ix) ; ";20H";
      If Unitx = Hilight Then Gosub Setrevers
      Print "Gear " ; Hex(chn) ; ":" ;
      If Unitx = Hilight Then Gosub Setnormal
      Ix = Msgentadd(unitx , Cmd_c_show , 0)
   Case Cmd_c_show:
      Currw = Gearsspeed(chn)
      Ix = Chn + 12
      Print "{027}[" ; Str(ix) ; ";29H" ; Str(currw) ; Spc(4);
   Case Cmd_c_decr:
      Currw = Gearsspeed(chn)
      If Currw > -1020 Then
         Currw = Currw - 10
         Gearsspeed(chn) = Currw
      End If
      Ix = Msgentadd(unitx , Cmd_c_set , 0)
   Case Cmd_c_incr:
      Currw = Gearsspeed(chn)
      If Currw < 1020 Then
         Currw = Currw + 10
         Gearsspeed(chn) = Currw
      End If
      Ix = Msgentadd(unitx , Cmd_c_set , 0)
   End Select
End Sub
'=========================================================================================
' Control Gears
'=========================================================================================
Gearscontrol1:
      Absi = Abs(gearsspeedleft)
      Pwm1b = Absi
      If Gearsspeedleft = 0 Then
         Portd.4 = 0                                        ' off
         Tempb = Msgentadd(outunit , Cmd_c_off , &B00000001)
      Else
         Portd.4 = 1                                        ' on
         Tempb = Msgentadd(outunit , Cmd_c_on , &B00000001)
         If Gearsspeedleft > 0 Then
            Portc.6 = 1                                     ' fwd
            Portc.7 = 0                                     ' fwd
         Else
            Portc.6 = 0                                     ' bck
            Portc.7 = 1                                     ' bck
         End If
      End If
      Return
'=========================================================================================
Gearscontrol2:
      Absi = Abs(gearsspeedrite)
      Pwm1a = Absi
      If Gearsspeedrite = 0 Then
         Portd.5 = 0                                        ' off
         Tempb = Msgentadd(outunit , Cmd_c_off , &B00000010)
      Else
         Portd.5 = 1                                        ' on
         Tempb = Msgentadd(outunit , Cmd_c_on , &B00000010)
         If Gearsspeedrite > 0 Then
            Portb.0 = 1                                     ' fwd
            Portb.1 = 0                                     ' fwd
         Else
            Portb.0 = 0                                     ' bck
            Portb.1 = 1                                     ' bck
         End If
      End If
      Return
'=========================================================================================




'=========================================================================================
' CLASS Stepper
'=========================================================================================
Sub Steppbuild()
   Gosub Unitcreate
   Unitrefcls(unitfill) = Cls_c_stepp
   Incr Steppfill
   Unitrefid(unitfill) = Steppfill
   Steppref(steppfill) = Unitfill
   Steppspeed(steppfill) = 0

'=========================================================================================
'  Stepper Motor Def
'=========================================================================================
   If Steppfill = 1 Then

Config Pind.6 = Output                                      'Schrittmotoren Ein/Aus

Config Pinc.4 = Output                                      'Schrittmotor Rechts Richtung
Config Pinc.2 = Output                                      'Schrittmotor Rechts Step

   Else
Config Pind.6 = Output                                      'Schrittmotoren Ein/Aus

Config Pinc.5 = Output                                      'Schrittmotor Links Richtung
Config Pinc.3 = Output                                      'Schrittmotor Links Step

   End If

End Sub
'=========================================================================================
Sub Stepp(byval Unitx As Byte , Byval Cmd As Byte , Byval Param As Word)
Local Ix As Byte
Local Chn As Byte
Local Currw As Integer
   Chn = Unitrefid(unitx)
   Select Case Cmd
   Case Cmd_c_setup:
      Ix = Msgentadd(unitx , Cmd_c_text , 0)
      Ix = Msgentadd(unitx , Cmd_c_set , 0)
      Print "Setup: " ; Str(unitx) ; Spc(6);
      Print "MOT " ;
      If Chn = 1 Then
            Print "Stepp-R:";
      Else
            Print "Stepp-L:" ;
      End If
      Print
   Case Cmd_c_set:
      Gosub Steppcontrol
      If Chn = 1 Then Gosub Steppcontrolrite
      If Chn = 2 Then Gosub Steppcontrolleft
      Ix = Msgentadd(unitx , Cmd_c_show , 0)
   Case Cmd_c_text:
      Ix = Chn + 14
      Print "{027}[" ; Str(ix) ; ";20H";
      If Unitx = Hilight Then Gosub Setrevers
      If Chn = 1 Then
            Print "Stepp-R:";
      Else
            Print "Stepp-L:" ;
      End If
      If Unitx = Hilight Then Gosub Setnormal
      Ix = Msgentadd(unitx , Cmd_c_show , 0)
   Case Cmd_c_show:
      Currw = Steppspeed(chn)
      Ix = Chn + 14
      Print "{027}[" ; Str(ix) ; ";29H" ; Str(currw) ; Spc(4);
   Case Cmd_c_decr:
      Currw = Steppspeed(chn)
      If Currw > -100 Then
         Currw = Currw - 5
         Steppspeed(chn) = Currw
      End If
      Ix = Msgentadd(unitx , Cmd_c_set , 0)
   Case Cmd_c_incr:
      Currw = Steppspeed(chn)
      If Currw < 100 Then
         Currw = Currw + 5
         Steppspeed(chn) = Currw
      End If
      Ix = Msgentadd(unitx , Cmd_c_set , 0)
   End Select
End Sub
'=========================================================================================
' Control stepper
'=========================================================================================
Stepptick1:
      If Steppcountleft > 0 Then
         Decr Steppcountleft
      Else
         If Steppspeedleft <> 0 Then
            Portc.3 = 0                                     ' clock low
            Absi = Abs(steppspeedleft)
            Steppcountleft = 100 - Absi
            Portc.3 = 1                                     ' clock high
         End If
      End If
      Return
Stepptick2:
      If Steppcountrite > 0 Then
         Decr Steppcountrite
      Else
         If Steppspeedrite <> 0 Then
            Portc.2 = 0                                     ' clock low
            Absi = Abs(steppspeedrite)
            Steppcountrite = 100 - Absi
            Portc.2 = 1                                     ' clock high
         End If
      End If
      Return
'=========================================================================================
Steppcontrol:
      If Steppspeedleft = 0 And Steppspeedrite = 0 Then
         Portd.6 = 0                                        ' OFF
         Tempb = Msgentadd(outunit , Cmd_c_off , &B00001100)
      End If
      Return
Steppcontrolleft:
      If Steppspeedleft <> 0 Then
         Portd.6 = 1                                        ' ON (both)
         Tempb = Msgentadd(outunit , Cmd_c_on , &B00000100)
         If Steppspeedleft > 0 Then
            Portc.5 = 0                                     ' fwd
         Else
            Portc.5 = 1                                     ' bck
         End If
      Else
         Tempb = Msgentadd(outunit , Cmd_c_off , &B00000100)
         Steppcountleft = 0
      End If
      Return
Steppcontrolrite:
      If Steppspeedrite <> 0 Then
         Portd.6 = 1                                        ' ON (both)
         Tempb = Msgentadd(outunit , Cmd_c_on , &B00001000)
         If Steppspeedrite > 0 Then
            Portc.4 = 0                                     ' fwd
         Else
            Portc.4 = 1                                     ' bck
         End If
      Else
         Tempb = Msgentadd(outunit , Cmd_c_off , &B00001000)
         Steppcountrite = 0
      End If
      Return
